	.MODEL	LARGE	; Define layout of memory
	.386			; Use 32-bit registers

bPtr		equ	byte ptr	; Create shorter names for use
wPtr		equ	word ptr	; with the 'ptr' keyword
dPtr		equ 	dword ptr
STACKSIZE	equ	4096
TRUE		equ	1
FALSE		equ	0
CR		equ	0Dh	; Carriage Return
LF		equ	0Ah	; Line Feed
ESCAPE		equ	01Bh	; Escape
NEGSIGN		equ     02Dh    ; Negative sign (-)
COMMA		equ	02Ch	; Comma
ZERO		equ	030h	; ASCII for 0
NINE		equ	039h	; ASCII for 9
MAXCHARS	equ	20	; Max length of input string
mLeft		equ	0
mRight		equ	1

		ASSUME	CS:_TEXT, DS:_DATA, SS:STACK, ES:NOTHING
			
_TEXT         SEGMENT WORD PUBLIC USE16 'CODE'

start:		xor	edx,edx
		mov	dx,_DATA	; Initialize data segment value
		mov	ds, dx		; ds is @DATA
		mov	dx,STACK
		mov	ss,dx		; Initialize stack segment
		mov	esp,STACKSIZE	; Initialize stack pointer
					; END OF PROGRAM INITIALIZATION
		call 	_init_graphics	; Initialize the graphics screen
		call 	_init_mouse	; Initialize the mouse
		call 	_show_mouse	; Start showing the mouse
		
mainLoop:
		
	; Check for right button click	
		mov 	ax,5h		; Function for button press info
		mov	bx,1		; Load button-id into bx
		int 	33h		; Call mouse functions
		cmp	ax,10b		; Is the right button down?
		je	progEnd		; If yes, end
		
buttonLoop:
	; Check for left button click
		cmp	click,1
		je	clickOn
		mov 	ax,5h		; Function for button press info
		mov	bx,0		; Load button-id into bx
		int 	33h		; Call mouse functions
		cmp	ax,1b		; Is the right button down?
		je	leftClick	; If yes, end	
		
clickOn:
	; Check for left button release
		mov 	ax,6h		; Function for button press info
		mov	bx,0		; Load button-id into bx
		int 	33h		; Call mouse functions
		cmp	ax,1b		; Is the left button down?
		je	leftRelease	; If yes, jump to leftClick
		jmp	mainloop
	
releaseOn:
		jmp	mainLoop
		
progEnd:	call 	_hide_mouse	; Stop showing the mouse
        	call 	_done_graphics	; Restore the graphics mode
quit:		mov 	ax,4C00h	; SET UP FOR PROGRAM END
		int 	21h		; TERMINATE PROGRAM
		
leftClick:
		cmp	click,1
		je	clickOn
		call	_hide_mouse
		mov	click,1
		push	ds
		push	offset str1
		call	_putstring
		add	esp,4
		call	_CRLF
		call	_show_mouse
		jmp	mainLoop	; Return to main program
		
leftRelease:	
		cmp	click,1
		jne	mainLoop
		call	_hide_mouse
		mov	release,1
		push	ds
		push	offset str2
		call	_putstring
		add	esp,4
		call	_CRLF
		call	_show_mouse
		mov	click,0
		jmp	mainLoop	; Jump to end of program
 
; Module Name: _putstring
; void putstring(const char *str)
; Description: Prints a NULL terminated character string 'pStr' to console.
;       Input: str - a pointer to the string to be printed.
;      Output: None.
; Stack Frame: [ebp] - old ebp
;              [ebp+4] - Return address
;              [ebp+8] - str
;              [ebp+10] - segment for str
;    Comments: eax and edx are not preserved.
_putstring    PROC    FAR
              push    ebp                ; Save old ebp to stack
              mov     ebp,esp            ; Set ebp to stack pointer
              push    ds                 ; Save ds to stack
              push    esi                ; Save esi to stack
              mov     dx,[ebp+10]        ; Get data segment
              mov     ds,dx              ; Move data segment to ds
              xor     esi,esi            ; Initialize esi
              mov     si,[ebp+8]         ; Set esi to start of str
                                         ; Beginning of procedure code.
startPutLoop: mov     dl,[esi]           ; Move next char in string to dl.
              mov     ah,2               ; Prepare to output char to screen.
              int     21h                ; Output character to screen.
              inc     esi                ; Increase character counter by 1.
              cmp     dl,0               ; Check for end of string.
              je      endPutLoop         ; If at end, jump out of loop.
              jmp     startPutLoop       ; Otherwise return to start of loop.
endPutLoop:
              pop     esi                ; Restore value of esi
              pop     ds                 ; Restore value of ds
              pop     ebp                ; Restore value of ebp
              ret
_putstring    ENDP

; Module Name: _getstring
; short getstring(char *dst, short maxchars)
; Description: Reads keyboard input from the console. Stops at maxchars
;              or when an [esc] is encountered.
;       Input: maxchars - a pointer to the number of keys to read.
;      Output: Returns -1 in ax if [esc] was entered, else 0
; Stack Frame: [ebp] - old ebp
;              [ebp+4] - Return address
;              [ebp+8] - offset of dst
;              [ebp+10] - segment for dst
;              [ebp+12] - maxchars
;    Comments: eax and edx are not preserved.
_getstring    PROC    FAR
              push    ebp                ; Save old ebp to stack
              mov     ebp,esp            ; Set ebp to stack pointer
              push    esi                ; Save esi to stack
              push    ecx                ; Save ecx to stack
              push    ds                 ; Save ds to stack
              mov     dx,[ebp+10]        ; Get data segmnt for dst
              mov     ds,dx
              xor     esi,esi            ; Initialize esi
              mov     si,[ebp+8]         ; Set esi to str
              xor     eax,eax            ; Initialize eax
              mov     ax,[ebp+12]        ; Set ax to maxchars
              mov     cl,al              ; Set cl to maxchars
              mov     ch,0               ; Initialize character counter
                                         ; Beginning of procedure code.
startGetLoop: mov     ah,8
              int     21h                ; Get keyboard input.
              cmp     al,ESCAPE          ; Check for ESC
              je      endProgram         ; Exit if ESC was pressed.
              ; Character validation
              ;cmp     al,0Dh             ; RETURN is OK
              ;je      validChar
              ;cmp     al,32              ; SPACE is OK
              ;je      validChar
              ;cmp     al,65              ; < 65 is not letter - ignore
              ;jl      startGetLoop
              ;cmp     al,122             ; > 122 is not letter - ignore
              ;jg      startGetLoop
              ;cmp     al,97              ; >=97 - must be lower case - OK
              ;jge     validChar
              ;cmp     al,90              ; <=90 - must be upper case - OK
              ;jg      startGetLoop
validChar:    mov     dl,al              ; Echo key to screen.
              mov     ah,2
              int     21h
              cmp     dl,0Dh             ; Check for RETURN
              je      endGetLoop         ; Exit if RETURN pressed.
              mov     [esi],dl           ; Add the character to the string.
              inc     esi                ; Add one to character counter.
              inc     ch                 ; Add one to character counter.
              cmp     cl,ch              ; Compare counter to maxchars.
              je      endGetLoop         ; Exit if maxchars is reached.
              jmp     startGetLoop       ; Get next character.
endGetLoop:   mov     dl,0
              mov     [esi],dl           ; Add a null terminator to string.
              mov     eax,0               ; Return value of function.
              jmp     return
endProgram:   mov     dl,0
              mov     [esi],dl           ; Add a null terminator to string.
              mov     eax,-1              ; Return value of function.
return:
              pop     ds                 ; Restore value of ds
              pop     ecx                ; Restore value of ecx
              pop     esi                ; Restore value of esi
              pop     ebp                ; Restore value of ebp
              ret
_getstring    ENDP

; Module Name: _CRLF
; void CRLF()
; Description: Prints an endline to console.
;       Input: None.
;      Output: None.
; Stack Frame: None.
;    Comments: eax and edx are not preserved.
_CRLF         PROC
              mov     dl,CR		; Write CR to console.
              mov     ah,2
              int     21h
              mov     dl,LF		; Write LF to console.
              mov     ah,2
              int     21h
              ret
_CRLF         ENDP


; Module Name: _init_mouse

; void mouse()

; Description: Initializes the mouse.

;       Input: None.

;      Output: None.

; Stack Frame: None.

;    Comments: The values in 'eax' and 'edx' are not preserved.

_init_mouse	PROC	FAR

		mov ax, 0		; Load mouse service for reset

		int 33h			; Print character

		ret			; End of _init_mouse

_init_mouse		ENDP



; Module Name: _show_mouse

; void show_mouse()

; Description: Shows the mouse.

;       Input: None.

;      Output: None.

; Stack Frame: None.

;    Comments: The values in 'eax' and 'edx' are not preserved.

_show_mouse	PROC	FAR

		mov ax, 1		; Load mouse service for reset

		int 33h			; Print character

		ret			; End of _initialize_mouse

_show_mouse		ENDP



; Module Name: _hide_mouse

; void hide_mouse()

; Description: Hides the mouse.

;       Input: None.

;      Output: None.

; Stack Frame: None.

;    Comments: The values in 'eax' and 'edx' are not preserved.

_hide_mouse	PROC	FAR

		mov ax, 2		; Load mouse service for reset

		int 33h			; Print character

		ret			; End of _initialize_mouse

_hide_mouse		ENDP



; Module Name: _init_graphics

; void _init_graphics()

; Description: Initializes the graphics screen.

;       Input: None.

;      Output: None.

; Stack Frame: None.

;    Comments: The values in 'eax' and 'edx' are not preserved.

_init_graphics	PROC	FAR

		push ebx		; Back up EBX register



		mov ah, 0fh		; Get current video mode

		int 10h			; Call function

		mov orig_vmode, al	; Store return in static variable

					

		mov ah, 0		; Set the video mode

		mov al, 13h		; Graphics mode 13h

		int 10h			; Call function

					

					; This routine should be replaced with 

					; a generic color indexer

					; .bmp file support would be useful first though

					

					; Set color zero (background) to blue

		mov dx, 3c8h			; Port for selecting index number

		mov al, 0				; Index 0

		out dx, al				; Send to port

					

		mov dx, 3c9h		; Port for selecting colors

		mov al, 0		; Red intensity

		out dx, al		; Send to port

		mov al, 0		; Green intensity

		out dx, al		; Send to port

		mov al, 35		; Blue intensity

		out dx, al		; Send to port

					

					; Set color one to white

		mov dx, 3c8h		; Port for selecting index number

		mov al, 1		; Index 0

		out dx, al		; Send to port

					

		mov dx, 3c9h		; Port for selecting colors

		mov al, 63		; Red intensity

		out dx, al		; Send to port

		mov al, 63		; Green intensity

		out dx, al		; Send to port

		mov al, 63		; Blue intensity

		out dx, al		; Send to port

					

		pop ebx			; Restore EBX register

		ret			; End of _init_graphics

_init_graphics	ENDP



; Module Name: _done_graphics

; void _done_graphics()

; Description: Restores the graphics screen.

;       Input: None.

;      Output: None.

; Stack Frame: None.

;    Comments: The values in 'eax' and 'edx' are not preserved.

_done_graphics	PROC	FAR

		mov ah, 0		; Set the video mode

		mov al, orig_vmode	; Original raphics mode

		int 10h			; Call function

		ret			; End of _done_graphics

_done_graphics	ENDP



_TEXT		ENDS



STACK		SEGMENT	PARA	STACK	USE16	'STACK'

programStack	db	STACKSIZE dup('*')

STACK		ENDS



_DATA		SEGMENT	WORD	PUBLIC	USE16	'DATA'	

orig_vmode	db	?

click		db	0

release		db	0

str1		db	"press",0

str2		db	"release",0

_DATA		ENDS



		END	start
